@@ -5,9 +5,10 @@ from typing import Dict, List, Set, Tuple
 
 import xml.dom.minidom
 
-from mir.tools import class_ids, mir_storage_ops
+from mir.tools import class_ids
 from mir.tools.code import MirCode
-from mir.tools.phase_logger import PhaseLoggerCenter, PhaseStateEnum
+from mir.tools.errors import MirRuntimeError
+from mir.tools.phase_logger import PhaseLoggerCenter
 from mir.protos import mir_command_pb2 as mirpb
 
 
@@ -26,7 +27,7 @@ def _get_dom_xml_tag_node(node: xml.dom.minidom.Element, tag_name: str) -> xml.d
     tag_nodes = node.getElementsByTagName(tag_name)
     if len(tag_nodes) > 0 and len(tag_nodes[0].childNodes) > 0:
         return tag_nodes[0]
-    raise KeyError(f"found no element for key: {tag_name}")
+    raise MirRuntimeError(MirCode.RC_CMD_ERROR_UNKNOWN, f"found no element for key: {tag_name}")
 
 
 def _get_dom_xml_tag_data(node: xml.dom.minidom.Element, tag_name: str) -> str:
@@ -103,7 +104,8 @@ def _read_customized_Keywords(ck_file: str) -> Dict[str, Set[str]]:
             continue
 
         if components[0] in name_cks:
-            raise ValueError(f'dumplicate asset name in ck file: {components[0]}')
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_MIR_FILE,
+                                  error_message=f'dumplicate asset name in ck file: {components[0]}')
 
         name_cks[components[0]] = set(components[1:])
 
@@ -147,7 +149,7 @@ def import_annotations(mir_annotation: mirpb.MirAnnotations, mir_keywords: mirpb
     assethash_filename_list = []  # type: List[Tuple[str, str, str]]
     with open(in_sha1_file, "r") as in_file:
         for line in in_file.readlines():
-            line_components = line.strip().split()
+            line_components = line.strip().split('\t')
             if not line_components or len(line_components) < 2:
                 logging.warning("incomplete line: %s", line)
                 continue
