@@ -5,14 +5,17 @@ exports the assets and annotations from mir format to ark-training-format
 from collections.abc import Collection
 from enum import Enum
 import logging
+import json
 import os
-from typing import Any, Dict, List, Optional, Set
+from typing import Any, Callable, Dict, List, Optional, Set
+import uuid
 import xml.etree.ElementTree as ElementTree
 
 from mir.protos import mir_command_pb2 as mirpb
 from mir.tools import class_ids, mir_storage_ops
 from mir.tools import utils as mir_utils
-from mir.tools.phase_logger import PhaseLoggerCenter, PhaseStateEnum
+from mir.tools.code import MirCode
+from mir.tools.errors import MirRuntimeError
 
 
 class ExportError(Exception):
@@ -27,17 +30,37 @@ class ExportFormat(str, Enum):
     EXPORT_FORMAT_NO_ANNOTATION = 'none'
     EXPORT_FORMAT_ARK = 'ark'
     EXPORT_FORMAT_VOC = 'voc'
+    EXPORT_FORMAT_LS_JSON = 'ls_json'  # label studio json format
 
 
-# public: format type
-SUPPORTED_EXPORT_FORMATS = {
-    ExportFormat.EXPORT_FORMAT_NO_ANNOTATION.value, ExportFormat.EXPORT_FORMAT_ARK.value,
-    ExportFormat.EXPORT_FORMAT_VOC.value
-}
+def check_support_format(anno_format: str) -> bool:
+    return anno_format in support_format_type()
 
 
-def format_type_from_str(format: str) -> ExportFormat:
-    return ExportFormat(format.lower())
+def support_format_type() -> List[str]:
+    return [f.value for f in ExportFormat]
+
+
+def format_type_from_str(anno_format: str) -> ExportFormat:
+    return ExportFormat(anno_format.lower())
+
+
+def format_file_ext(anno_format: ExportFormat) -> str:
+    _format_ext_map = {
+        ExportFormat.EXPORT_FORMAT_ARK: '.txt',
+        ExportFormat.EXPORT_FORMAT_VOC: '.xml',
+        ExportFormat.EXPORT_FORMAT_LS_JSON: '.json',
+    }
+    return _format_ext_map[anno_format]
+
+
+def format_file_output_func(anno_format: ExportFormat) -> Callable:
+    _format_func_map = {
+        ExportFormat.EXPORT_FORMAT_ARK: _single_image_annotations_to_ark,
+        ExportFormat.EXPORT_FORMAT_VOC: _single_image_annotations_to_voc,
+        ExportFormat.EXPORT_FORMAT_LS_JSON: _single_image_annotations_to_ls_json,
+    }
+    return _format_func_map[anno_format]
 
 
 # public: export
@@ -50,6 +73,7 @@ def export(mir_root: str,
            need_ext: bool,
            need_id_sub_folder: bool,
            base_branch: str,
+           base_task_id: str,
            format_type: ExportFormat,
            index_file_path: str = None,
            index_prefix: str = None) -> bool:
@@ -72,13 +96,20 @@ def export(mir_root: str,
         index_prefix (str | None): prefix added to each line in index path
 
     Raises:
-        ValueError: if mir repo not provided
+        MirRuntimeError
 
     Returns:
         bool: returns True if success
     """
     if not mir_root:
-        raise ValueError("invalid mir_repo")
+        raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
+                              error_message="invalid mir_repo",
+                              needs_new_commit=False)
+
+    if not check_support_format(format_type):
+        raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
+                              error_message=f"invalid --format: {format_type}",
+                              needs_new_commit=False)
 
     # export assets
     os.makedirs(asset_dir, exist_ok=True)
@@ -103,6 +134,7 @@ def export(mir_root: str,
     mir_datas = mir_storage_ops.MirStorageOps.load(
         mir_root=mir_root,
         mir_branch=base_branch,
+        mir_task_id=base_task_id,
         mir_storages=[mirpb.MirStorage.MIR_METADATAS, mirpb.MirStorage.MIR_ANNOTATIONS])
     mir_annotations: mirpb.MirAnnotations = mir_datas[mirpb.MirStorage.MIR_ANNOTATIONS]
 
@@ -113,20 +145,15 @@ def export(mir_root: str,
         class_type_ids=set(class_type_ids.keys()) if class_type_ids else None,
         base_task_id=mir_annotations.head_task_id)
 
-    if format_type == ExportFormat.EXPORT_FORMAT_ARK:
-        _export_detect_ark_annotations_to_path(annotations_dict=assets_to_det_annotations_dict,
-                                               asset_ids=list(asset_ids),
-                                               dest_path=annotation_dir,
-                                               class_type_mapping=class_type_ids)
-    elif format_type == ExportFormat.EXPORT_FORMAT_VOC:
-        mir_metadatas: mirpb.MirMetadatas = mir_datas[mirpb.MirStorage.MIR_METADATAS]
-        _export_detect_voc_annotations_to_path(annotations_dict=assets_to_det_annotations_dict,
-                                               mir_metadatas=mir_metadatas,
-                                               asset_ids=list(asset_ids),
-                                               dest_path=annotation_dir,
-                                               mir_root=mir_root)
-    else:
-        raise ValueError(f"unsupported format: {format_type.name}")
+    mir_metadatas: mirpb.MirMetadatas = mir_datas[mirpb.MirStorage.MIR_METADATAS]
+    _export_detect_annotations_to_path(asset_ids=list(asset_ids),
+                                       format_type=format_type,
+                                       mir_metadatas=mir_metadatas,
+                                       annotations_dict=assets_to_det_annotations_dict,
+                                       class_type_mapping=class_type_ids,
+                                       dest_path=annotation_dir,
+                                       mir_root=mir_root,
+                                       assert_id_filename_map=asset_result)
 
     return True
 
@@ -149,12 +176,14 @@ def _generate_asset_index_file(asset_rel_paths: Collection,
         FileExistsError: if index file already exists, and override set to False
     """
     if not asset_rel_paths or not index_file_path:
-        raise ValueError('empty asset_rel_paths or index_file_path')
+        raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
+                              error_message='empty asset_rel_paths or index_file_path')
     if os.path.exists(index_file_path):
         if overwrite:
             logging.warning(f"index file already exists, overwriting: {index_file_path}")
         else:
-            raise FileExistsError(f"index file already exists: {index_file_path}")
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
+                                  error_message=f"index file already exists: {index_file_path}")
 
     with open(index_file_path, 'w') as f:
         for item in asset_rel_paths:
@@ -181,7 +210,8 @@ def _annotations_by_assets(mir_annotations: mirpb.MirAnnotations, class_type_ids
     assets_to_det_annotations_dict = {}  # type: Dict[str, List[mirpb.Annotation]]
 
     if base_task_id not in mir_annotations.task_annotations:
-        raise ValueError(f"base task id: {base_task_id} not in mir_annotations")
+        raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_MIR_REPO,
+                              error_message=f"base task id: {base_task_id} not in mir_annotations")
 
     task_annotations = mir_annotations.task_annotations[base_task_id]
     for asset_id, image_annotations in task_annotations.image_annotations.items():
@@ -194,43 +224,15 @@ def _annotations_by_assets(mir_annotations: mirpb.MirAnnotations, class_type_ids
     return assets_to_det_annotations_dict
 
 
-# private: export annotations: ark
-def _export_detect_ark_annotations_to_path(annotations_dict: Dict[str, List[mirpb.Annotation]], asset_ids: List[str],
-                                           dest_path: str, class_type_mapping: Optional[Dict[int, int]]) -> None:
+def _export_detect_annotations_to_path(asset_ids: List[str], format_type: ExportFormat,
+                                       mir_metadatas: mirpb.MirMetadatas,
+                                       annotations_dict: Dict[str, List[mirpb.Annotation]],
+                                       class_type_mapping: Optional[Dict[int, int]], dest_path: str, mir_root: str,
+                                       assert_id_filename_map: Dict[str, str]) -> None:
     if not asset_ids:
-        raise ValueError("empty asset ids")
-
-    os.makedirs(dest_path, exist_ok=True)
-
-    missing_counter = 0
-    empty_counter = 0
-    for asset_id in asset_ids:
-        # calc missing or empty
-        if asset_id not in annotations_dict:
-            missing_counter += 1
-            continue
-        annotations = annotations_dict[asset_id]
-        if len(annotations) == 0:
-            empty_counter += 1
-            continue
-
-        file_name = f"{asset_id}.txt"
-        dest_file_path = os.path.join(dest_path, file_name)
-        _single_image_annotations_to_ark(annotations, dest_file_path, class_type_mapping)
-
-    if missing_counter > 0 or empty_counter > 0:
-        logging.warning(f"missing annotations assets: {missing_counter}, "
-                        f"empty annotations assets: {empty_counter} out of {len(asset_ids)}")
-
-
-# private: export annotations: voc
-def _export_detect_voc_annotations_to_path(annotations_dict: Dict[str, List[mirpb.Annotation]],
-                                           mir_metadatas: mirpb.MirMetadatas, asset_ids: List[str],
-                                           dest_path: str, mir_root: str) -> None:
-    if not asset_ids:
-        raise ValueError('empty asset_ids')
+        raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS, error_message='empty asset_ids')
     if not mir_metadatas:
-        raise ValueError('invalid mir_metadatas')
+        raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS, error_message='invalid mir_metadatas')
 
     os.makedirs(dest_path, exist_ok=True)
 
@@ -240,38 +242,46 @@ def _export_detect_voc_annotations_to_path(annotations_dict: Dict[str, List[mirp
     empty_counter = 0
     for asset_id in asset_ids:
         if asset_id not in mir_metadatas.attributes:
-            raise ValueError(f"can not find asset id: {asset_id} in mir_metadatas")
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_MIR_FILE,
+                                  error_message=f"can not find asset id: {asset_id} in mir_metadatas")
+        attrs = mir_metadatas.attributes[asset_id]
 
         if asset_id not in annotations_dict:
             missing_counter += 1
-            continue
-        annotations = annotations_dict[asset_id]
+            annotations = []
+        else:
+            annotations = annotations_dict[asset_id]
         if len(annotations) == 0:
             empty_counter += 1
-            continue
 
-        attrs = mir_metadatas.attributes[asset_id]
-        annotations = annotations_dict[asset_id]
+        format_func = format_file_output_func(anno_format=format_type)
+        anno_str = format_func(asset_id=asset_id,
+                               attrs=attrs,
+                               annotations=annotations,
+                               class_type_mapping=class_type_mapping,
+                               cls_id_mgr=cls_id_mgr,
+                               asset_filename=assert_id_filename_map[asset_id])
+        with open(os.path.join(dest_path, f"{asset_id}{format_file_ext(format_type)}"), 'w') as f:
+            f.write(anno_str)
 
-        file_name = f"{asset_id}.xml"
-        dest_file_path = os.path.join(dest_path, file_name)
-        _single_image_annotations_to_voc(asset_id, attrs, annotations, dest_file_path, cls_id_mgr)
-
-    if missing_counter > 0:
-        logging.warning(f"missing annotations assets: {missing_counter}, empty annotations assets: {empty_counter}")
+    logging.info(f"missing annotations: {missing_counter}, "
+                 f"empty annotations: {empty_counter} out of {len(asset_ids)} assets")
 
 
-def _single_image_annotations_to_ark(annotations: List[mirpb.Annotation], dest_file_path: str,
-                                     class_type_mapping: Optional[Dict[int, int]]) -> None:
-    with open(dest_file_path, 'w') as f:
-        for annotation in annotations:
-            mapped_id = class_type_mapping[annotation.class_id] if class_type_mapping else annotation.class_id
-            f.write(f"{mapped_id}, {annotation.box.x}, {annotation.box.y}, "
-                    f"{annotation.box.x + annotation.box.w - 1}, {annotation.box.y + annotation.box.h - 1}\n")
+def _single_image_annotations_to_ark(asset_id: str, attrs: Any, annotations: List[mirpb.Annotation],
+                                     class_type_mapping: Optional[Dict[int, int]], cls_id_mgr: class_ids.ClassIdManager,
+                                     asset_filename: str) -> str:
+    output_str = ""
+    for annotation in annotations:
+        mapped_id = class_type_mapping[annotation.class_id] if class_type_mapping else annotation.class_id
+        output_str += f"{mapped_id}, {annotation.box.x}, {annotation.box.y}, "
+        output_str += f"{annotation.box.x + annotation.box.w - 1}, {annotation.box.y + annotation.box.h - 1}\n"
+    return output_str
 
 
 def _single_image_annotations_to_voc(asset_id: str, attrs: Any, annotations: List[mirpb.Annotation],
-                                     dest_file_path: str, cls_id_mgr: class_ids.ClassIdManager) -> None:
+                                     class_type_mapping: Optional[Dict[int, int]], cls_id_mgr: class_ids.ClassIdManager,
+                                     asset_filename: str) -> str:
     # annotation
     annotation_node = ElementTree.Element('annotation')
 
@@ -281,7 +291,7 @@ def _single_image_annotations_to_voc(asset_id: str, attrs: Any, annotations: Lis
 
     # annotation: filename
     filename_node = ElementTree.SubElement(annotation_node, 'filename')
-    filename_node.text = asset_id
+    filename_node.text = asset_filename
 
     # annotation: source
     source_node = ElementTree.SubElement(annotation_node, 'source')
@@ -350,6 +360,47 @@ def _single_image_annotations_to_voc(asset_id: str, attrs: Any, annotations: Lis
         difficult_node = ElementTree.SubElement(object_node, 'difficult')
         difficult_node.text = '0'
 
-    # write to xml
-    tree = ElementTree.ElementTree(annotation_node)
-    tree.write(dest_file_path, encoding='utf-8')
+    return ElementTree.tostring(element=annotation_node, encoding='unicode')
+
+
+def _single_image_annotations_to_ls_json(asset_id: str, attrs: Any, annotations: List[mirpb.Annotation],
+                                         class_type_mapping: Optional[Dict[int, int]],
+                                         cls_id_mgr: class_ids.ClassIdManager, asset_filename: str) -> str:
+    out_type = "predictions"  # out_type: annotation type - "annotations" or "predictions"
+    to_name = 'image'  # to_name: object name from Label Studio labeling config
+    from_name = 'label'  # control tag name from Label Studio labeling config
+    task: Dict = {
+        out_type: [{
+            "result": [],
+            "ground_truth": False,
+        }],
+        "data": {
+            "image": asset_filename
+        }
+    }
+
+    for annotation in annotations:
+        bbox_x, bbox_y = float(annotation.box.x), float(annotation.box.y)
+        bbox_width, bbox_height = float(annotation.box.w), float(annotation.box.h)
+        img_width, img_height = attrs.width, attrs.height
+        item = {
+            "id": uuid.uuid4().hex[0:10],  # random id to identify this annotation.
+            "type": "rectanglelabels",
+            "value": {
+                # Units of image annotations in label studio is percentage of image width/height.
+                # https://labelstud.io/guide/predictions.html#Units-of-image-annotations
+                "x": bbox_x / img_width * 100,
+                "y": bbox_y / img_height * 100,
+                "width": bbox_width / img_width * 100,
+                "height": bbox_height / img_height * 100,
+                "rotation": 0,
+                "rectanglelabels": [cls_id_mgr.main_name_for_id(annotation.class_id) or 'unknown']
+            },
+            "to_name": to_name,
+            "from_name": from_name,
+            "image_rotation": 0,
+            "original_width": img_width,
+            "original_height": img_height
+        }
+        task[out_type][0]['result'].append(item)
+    return json.dumps(task)
